/* Name: main.c
 * Project: Data input based on AVR USB driver with ...
 * 
 * Author: Jacques Lepot
 * Creation Date: 2008-04-19
 * Copyright: (c) 2008 by Jacques LEpot
 * License: Proprietary, free under certain conditions.
*** ^^^^ BLA BLA BLA
* modifikacijos by SAVEL. 

 */


#define F_CPU 12000000L

#include <avr/io.h>
#include <avr/wdt.h>

#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>


#include "usbdrv/oddebug.h"
#include "usbdrv/usbdrv.h"



#define BIT_LED 1
#define BIT_KEY 4


#define UTIL_BIN4(x)        (uchar)((0##x & 01000)/64 + (0##x & 0100)/16 + (0##x & 010)/4 + (0##x & 1))
#define UTIL_BIN8(hi, lo)   (uchar)(UTIL_BIN4(hi) * 16 + UTIL_BIN4(lo))

#ifndef NULL
#define NULL    ((void *)0)
#endif

/* ------------------------------------------------------------------------- */

static uchar    reportBuffer[5];    /* buffer for HID reports, type game cntrler*/
static uchar    idleRate;           /* in 4 ms units */

static uchar    adcPending;
static uchar    isRecording;
static uchar    adchanel;
static int		adcval1;
static int		adcval2;

/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { /* USB report descriptor */
    0x05, 0x01,        // USAGE_PAGE (Generic Desktop = 01)
	0x09, 0x05,        // USAGE (Game Pad = 05)
	0xa1, 0x01,        // COLLECTION (Application)
	0x09, 0x01,        //   USAGE (Pointer)
	0xa1, 0x00,        //   COLLECTION (Physical)
	0x09, 0x30,        //     USAGE (X)
	0x09, 0x31,        //     USAGE (Y)
	0x15, 0x00,        //   LOGICAL_MINIMUM (0)
	0x26, 0xff, 0x00,  //     LOGICAL_MAXIMUM (255)
	0x75, 0x08,        //   REPORT_SIZE (8bits)
	0x95, 0x04,        //   REPORT_COUNT (2)
	0x81, 0x02,        //   INPUT (Data,Var,Abs)
	0xc0,              // END_COLLECTION
	
	0x05, 0x09,        // USAGE_PAGE (Button)
	0x19, 0x01,        //   USAGE_MINIMUM (Button 1)
	0x29, 0x08,        //   USAGE_MAXIMUM (Button 8)
	0x15, 0x00,        //   LOGICAL_MINIMUM (0)
	0x25, 0x01,        //   LOGICAL_MAXIMUM (1)
	0x75, 0x01,        // REPORT_SIZE (1bit)
	0x95, 0x08,        // REPORT_COUNT (8)
	0x81, 0x02,        // INPUT (Data,Var,Abs)
	0xc0               // END_COLLECTION
};

/* ------------------------------------------------------------------------- */

static void buildReport(void)
{

	reportBuffer[0] = adcval1>>8;
	reportBuffer[1] = adcval1;
	reportBuffer[2] = adcval2>>8;
	reportBuffer[3] = adcval2;
	reportBuffer[4] = 0x01;

}



/* ------------------------------------------------------------------------- */

static void adcPoll(void)
{
int value;
// Select pin ADC0 using MUX
			ADMUX = 0 | _BV(REFS1) | _BV(REFS0);
			
			ADCSRA |=_BV(ADSC); // ADC start
			loop_until_bit_is_set(ADCSRA, ADIF); //uzsiciklinam kol nebaigs darba ADC. Nekulturinga, bet bus gerai.
			value=ADCW;
			adcval1 = value + value + (value >> 1); // kad gauti tiesiogiai mV, adcval=2.5*value
			
			ADMUX = 1 | _BV(REFS1) | _BV(REFS0);
			

	
			ADCSRA |=_BV(ADSC);
			loop_until_bit_is_set(ADCSRA, ADIF); //uzsiciklinam kol nebaigs darba ADC. Nekulturinga, bet bus gerai.
			value=ADCW;
			adcval2 = value + value + (value >> 1); // kad gauti tiesiogiai mV, adcval=2.5*value

}



/* ------------------------------------------------------------------------- */


static void adcInit(void)
{
    ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
}

/* ------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* ------------------------------------------------------------------------- */

uchar	usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    usbMsgPtr = reportBuffer;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
												   /* we only have one report type, so don't look at wValue */
            buildReport();
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
	return 0;
}



void    usbEventResetReady(void)
{

}

/* ------------------------------------------------------------------------- */
/* --------------------------------- main ---------------------------------- */
/* ------------------------------------------------------------------------- */

int main(void)
{
uchar   i;
uchar   calibrationValue;


    odDebugInit();
    usbDeviceDisconnect();
    for(i=0;i<20;i++){  /* 300 ms disconnect */
        _delay_ms(15);
    }
    usbDeviceConnect();

    wdt_enable(WDTO_1S);
    adcInit();
    usbInit();
    sei();

    for(;;){    /* main event loop */
        wdt_reset();
        usbPoll();
        if(usbInterruptIsReady() ){ /* we can send a report */
            buildReport();
            usbSetInterrupt(reportBuffer, sizeof(reportBuffer));

        }
        //do something
        adcPoll();
    }
    return 0;
}
